=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2009-03-16 04:48:09 +0000
+++ bzrlib/smart/request.py	2009-03-18 04:49:31 +0000
@@ -27,6 +27,7 @@
 """
 
 import tempfile
+import threading
 
 from bzrlib import (
     bzrdir,
@@ -42,6 +43,10 @@
 """)
 
 
+jail_info = threading.local()
+jail_info.transports = None
+
+
 class SmartServerRequest(object):
     """Base class for request handlers.
 
@@ -121,6 +126,12 @@
         self._body_chunks = None
         return self.do_body(body_bytes)
 
+    def setup_jail(self):
+        jail_info.transports = [self._backing_transport]
+
+    def teardown_jail(self):
+        jail_info.transports = None
+
     def translate_client_path(self, client_path):
         """Translate a path received from a network client into a local
         relpath.
@@ -277,7 +288,11 @@
         # XXX: most of this error conversion is VFS-related, and thus ought to
         # be in SmartServerVFSRequestHandler somewhere.
         try:
-            return callable(*args, **kwargs)
+            self._command.setup_jail()
+            try:
+                return callable(*args, **kwargs)
+            finally:
+                self._command.teardown_jail()
         except (KeyboardInterrupt, SystemExit):
             raise
         except Exception, err:

=== modified file 'bzrlib/tests/test_smart_request.py'
--- bzrlib/tests/test_smart_request.py	2009-03-16 04:48:09 +0000
+++ bzrlib/tests/test_smart_request.py	2009-03-18 04:49:31 +0000
@@ -16,9 +16,11 @@
 
 """Tests for smart server request infrastructure (bzrlib.smart.request)."""
 
+import threading
+
 from bzrlib import errors
 from bzrlib.smart import request
-from bzrlib.tests import TestCase
+from bzrlib.tests import TestCase, TestCaseWithMemoryTransport
 
 
 class NoBodyRequest(request.SmartServerRequest):
@@ -61,6 +63,22 @@
         raise errors.NoSuchFile('xyzzy')
 
 
+class CheckJailRequest(request.SmartServerRequest):
+
+    def __init__(self, *args):
+        request.SmartServerRequest.__init__(self, *args)
+        self.jail_transports_log = []
+
+    def do(self):
+        self.jail_transports_log.append(request.jail_info.transports)
+
+    def do_chunk(self, bytes):
+        self.jail_transports_log.append(request.jail_info.transports)
+
+    def do_end(self):
+        self.jail_transports_log.append(request.jail_info.transports)
+
+
 class TestSmartRequest(TestCase):
 
     def test_request_class_without_do_body(self):
@@ -76,6 +94,20 @@
         handler.end_received()
         # Request done, no exception was raised.
 
+    def test_only_request_code_is_jailed(self):
+        transport = 'dummy transport'
+        handler = request.SmartServerRequestHandler(
+            transport, {'foo': CheckJailRequest}, '/')
+        handler.args_received(('foo',))
+        self.assertEqual(None, request.jail_info.transports)
+        handler.accept_body('bytes')
+        self.assertEqual(None, request.jail_info.transports)
+        handler.end_received()
+        self.assertEqual(None, request.jail_info.transports)
+        self.assertEqual(
+            [[transport]] * 3, handler._command.jail_transports_log)
+
+
 
 class TestSmartRequestHandlerErrorTranslation(TestCase):
     """Tests that SmartServerRequestHandler will translate exceptions raised by
@@ -134,3 +166,16 @@
             ('TokenMismatch', 'some-token', 'actual-token'),
             errors.TokenMismatch('some-token', 'actual-token'))
 
+
+class TestRequestJail(TestCaseWithMemoryTransport):
+    
+    def test_jail(self):
+        transport = self.get_transport('blah')
+        req = request.SmartServerRequest(transport)
+        tls = threading.local()
+        self.assertEqual(None, request.jail_info.transports)
+        req.setup_jail()
+        self.assertEqual([transport], request.jail_info.transports)
+        req.teardown_jail()
+        self.assertEqual(None, request.jail_info.transports)
+

